SystemOrganization addCategory: #'Spec-Widgets'!
SystemOrganization addCategory: #'Spec-Widgets-PolyWidgets'!
SystemOrganization addCategory: #'Spec-Widgets-Utilities'!

PanelMorph subclass: #PanelMorphWithSplitters
	instanceVariableNames: 'paneMorphs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-Utilities'!

!PanelMorphWithSplitters commentStamp: '<historical>' prior: 0!
A PanelMorphWithSplitters is which add splitters the way SystemWindow do it!

!PanelMorphWithSplitters methodsFor: 'submorphs-add/remove' stamp: 'BenjaminVanRyseghem 3/23/2012 17:00'!
addMorph: aMorph

	paneMorphs add: aMorph.
	^ super addMorph: aMorph! !

!PanelMorphWithSplitters methodsFor: 'submorphs-add/remove' stamp: ''!
addMorph: aMorph fullFrame: aLayout

	paneMorphs add: aMorph.
	super addMorph: aMorph fullFrame: aLayout! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: ''!
addPaneHSplitterBetween: topMorphs and: bottomMorphs
	"Add a horizontal splitter for the given morphs that share a common bottom fraction.
	If there is a horizontal discontinuity apply the splitter to the first contiguous group.
	Answer the morphs to which the splitter was applied."
	
	|targetY fixed rightFraction leftFrame rightFrame sorted morph topGroup bottomGroup splitter offset|
	topMorphs ifEmpty: [^self].
	targetY := topMorphs first layoutFrame bottomFraction.
	fixed := topMorphs select: [:m | m layoutFrame topFraction = m layoutFrame bottomFraction].
		"fixed morphs appear in both top and bottom"
	sorted := ((topMorphs reject: [:m | m layoutFrame topFraction = m layoutFrame bottomFraction])
		asSortedCollection: [:a :b | a layoutFrame rightFraction = b layoutFrame rightFraction
			ifTrue: [a layoutFrame leftFraction <= b layoutFrame leftFraction]
			ifFalse: [a layoutFrame rightFraction <= b layoutFrame rightFraction]]) readStream.
	sorted contents ifEmpty: [^fixed].
	topGroup := OrderedCollection new.
	rightFraction := sorted contents first layoutFrame leftFraction.
	[sorted atEnd or: [morph := sorted next.
			(morph layoutFrame leftFraction ~= rightFraction and: [
				morph layoutFrame rightFraction ~= rightFraction])]] whileFalse: [
		topGroup add: morph.
		rightFraction := morph layoutFrame rightFraction].
	leftFrame := topGroup first layoutFrame.
	rightFrame := topGroup last layoutFrame.
	bottomGroup := (bottomMorphs 
			reject: [:m | m layoutFrame topFraction = m layoutFrame bottomFraction])
			select: [:m |
		(m layoutFrame leftFraction
			between: leftFrame leftFraction
			and: rightFrame rightFraction) or: [
		m layoutFrame rightFraction
			between: leftFrame leftFraction
			and: rightFrame rightFraction]].
	offset := (topGroup collect: [:m | m layoutFrame bottomOffset ifNil: [0]]) max.
	splitter := ProportionalSplitterMorph new
		beSplitsTopAndBottom.
	splitter layoutFrame: (LayoutFrame
		fractions: (leftFrame leftFraction @ targetY
					corner: rightFrame rightFraction @ targetY)
		offsets: (((leftFrame leftOffset ifNil: [0]) @ 0 corner: ((rightFrame rightOffset ifNil: [0])@ 4))
		 	translateBy: 0 @ offset)).
	topGroup := topGroup, fixed.
	topGroup do: [:m | splitter addLeftOrTop: m].
	bottomGroup do: [:m | splitter addRightOrBottom: m].
	self addMorphBack: splitter.
	^topGroup! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: ''!
addPaneHSplitters
	"Add the horizontal pane splitters."

	|remaining targetBottom sameBottom sameTop|
	remaining := paneMorphs reject: [:each |
			each layoutFrame bottomFraction = 1 or: [
				each layoutFrame bottomFraction = 0]].
	[remaining notEmpty] whileTrue: [
		targetBottom := remaining first layoutFrame bottomFraction.
		sameBottom := remaining select: [:each |
			each layoutFrame bottomFraction = targetBottom].
		sameTop := paneMorphs select: [:each |
			each layoutFrame topFraction = targetBottom].
		remaining := remaining
			copyWithoutAll: (self addPaneHSplitterBetween: sameBottom and: sameTop)]! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: 'BenjaminVanRyseghem 3/23/2012 17:01'!
addPaneSplitters
	"Add the vertical and horizontal pane splitters."

	self removePaneSplitters.
	self addPaneVSplitters.
	self addPaneHSplitters.
	self linkSplittersToSplitters! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: ''!
addPaneVSplitterBetween: leftMorphs and: rightMorphs
	"Add a vertical splitter for the given morphs that share a common right fraction.
	If there is a vertical discontinuity apply the splitter to the first contiguous group.
	Answer the morphs to which the splitter was applied."
	
	|targetX fixed bottomFraction topFrame bottomFrame sorted morph leftGroup rightGroup splitter offset|
	leftMorphs ifEmpty: [^self].
	targetX := leftMorphs first layoutFrame rightFraction.
	fixed := leftMorphs select: [:m | m layoutFrame leftFraction = m layoutFrame rightFraction].
		"fixed morphs appear in both top and bottom"
	sorted := ((leftMorphs reject: [:m | m layoutFrame leftFraction = m layoutFrame rightFraction])
		asSortedCollection: [:a :b | a layoutFrame bottomFraction = b layoutFrame bottomFraction
			ifTrue: [a layoutFrame topFraction <= b layoutFrame topFraction]
			ifFalse: [a layoutFrame bottomFraction <= b layoutFrame bottomFraction]]) readStream.
	sorted contents ifEmpty: [^fixed].
	leftGroup := OrderedCollection new.
	bottomFraction := sorted contents first layoutFrame topFraction.
	[sorted atEnd or: [morph := sorted next.
			morph layoutFrame topFraction ~= bottomFraction and: [
				morph layoutFrame bottomFraction ~= bottomFraction]]] whileFalse: [
		leftGroup add: morph.
		bottomFraction := morph layoutFrame bottomFraction].
	topFrame := leftGroup first layoutFrame.
	bottomFrame := leftGroup last layoutFrame.
	rightGroup := (rightMorphs
			reject: [:m | m layoutFrame leftFraction = m layoutFrame rightFraction])
			select: [:m |
		m layoutFrame topFraction
			between: topFrame topFraction
			and: bottomFrame bottomFraction].
	offset := (leftGroup collect: [:m | m layoutFrame rightOffset ifNil: [0]]) max.
	splitter := ProportionalSplitterMorph new.
	splitter layoutFrame: (LayoutFrame
		fractions: (targetX @ topFrame topFraction
					corner: targetX @ bottomFrame bottomFraction)
		offsets: ((0 @ (topFrame topOffset ifNil: [0]) corner: (4@ (bottomFrame bottomOffset ifNil: [0])))
		 	translateBy: offset @ 0)).
	leftGroup := leftGroup, fixed.
	leftGroup do: [:m | splitter addLeftOrTop: m].
	rightGroup do: [:m | splitter addRightOrBottom: m].
	self addMorphBack: splitter.
	^leftGroup! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: ''!
addPaneVSplitters
	"Add the vertical pane splitters."

	|remaining targetRight sameRight sameLeft |
	remaining := paneMorphs reject: [:each |
			each layoutFrame rightFraction = 1 or: [
				each layoutFrame rightFraction = 0]].
	[remaining notEmpty] whileTrue: [
		targetRight := remaining first layoutFrame rightFraction.
		sameRight := remaining select: [:each |
			each layoutFrame rightFraction = targetRight].
		sameLeft := paneMorphs select: [:each |
			each layoutFrame leftFraction = targetRight and: [
				each layoutFrame rightFraction ~= targetRight]].
		remaining := remaining
			copyWithoutAll: (self addPaneVSplitterBetween: sameRight and: sameLeft)]! !

!PanelMorphWithSplitters methodsFor: 'initialization' stamp: ''!
initialize
	"Initialization code for PanelMorphWithSplitter"

	super initialize.

	paneMorphs := OrderedCollection new.! !

!PanelMorphWithSplitters methodsFor: 'layout-properties' stamp: ''!
layoutFrame: aFrame

	self activate.
	super layoutFrame: aFrame! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: ''!
linkSplittersToSplitters
	"The pane morphs are already linked. Cross link the splitters as appropriate."
	
	self splitters do: [:each |
		each splitsTopAndBottom
			ifTrue: [self splitters do: [:eachMorph |
					eachMorph splitsTopAndBottom ~= each splitsTopAndBottom ifTrue: [
						eachMorph layoutFrame bottomFraction = each layoutFrame topFraction
							ifTrue: [each addLeftOrTop: eachMorph].
						eachMorph layoutFrame topFraction = each layoutFrame bottomFraction
							ifTrue: [each addRightOrBottom: eachMorph]]]]
			ifFalse: [self splitters do: [:eachMorph |
					eachMorph splitsTopAndBottom ~= each splitsTopAndBottom ifTrue: [
						eachMorph layoutFrame rightFraction = each layoutFrame leftFraction
							ifTrue: [each addLeftOrTop: eachMorph].
						eachMorph layoutFrame leftFraction = each layoutFrame rightFraction
							ifTrue: [each addRightOrBottom: eachMorph]]]].
				
		each comeToFront.]! !

Object subclass: #MorphWrapper
	instanceVariableNames: 'layout morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-Utilities'!

!MorphWrapper commentStamp: '<historical>' prior: 0!
A MorphWrapper is used to wrap a morph with a layout in the goal to be easily added to another morph!

!MorphWrapper class methodsFor: 'instance creation' stamp: ''!
morph: morph layout: layout 

	^ self new
		layout: layout;
		morph: morph;
		yourself! !

!MorphWrapper methodsFor: 'adding' stamp: ''!
addIn: aContainer
		
	aContainer
		addMorph: morph
		fullFrame: layout! !

!MorphWrapper methodsFor: 'accessing' stamp: ''!
frame: relFrame
	| frame |
	frame := LayoutFrame new.
	frame 
		leftFraction: relFrame left; 
		rightFraction: relFrame right; 
		topFraction: relFrame top; 
		bottomFraction: relFrame bottom.
	layout := frame! !

!MorphWrapper methodsFor: 'accessing' stamp: ''!
fullFrame: anObject
	
	layout := anObject! !

!MorphWrapper methodsFor: 'accessing' stamp: ''!
morph: anObject
	
	morph := anObject! !

Object subclass: #RadioButtonGroup
	instanceVariableNames: 'buttons'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-Utilities'!

!RadioButtonGroup commentStamp: '<historical>' prior: 0!
A RadioButtonGroup is a group for radio buttons which ensure that only radio button of this group is activated at the same time.!

!RadioButtonGroup methodsFor: 'protocol' stamp: ''!
addRadioButton: aButton

	buttons do: [:b | 
		aButton whenActivatedDo: [ b privateSetState: false ].
		b whenActivatedDo: [ aButton privateSetState: false ]].
		
	buttons add: aButton.
	aButton privateSetState: false! !

!RadioButtonGroup methodsFor: 'protocol' stamp: ''!
default: aButton

	aButton privateSetState: true.
	aButton takeKeyboardFocus.! !

!RadioButtonGroup methodsFor: 'initialization' stamp: ''!
initialize
	"Initialization code for RadioButtonGroup"

	super initialize.

	buttons := OrderedCollection new.! !

ComposableModel subclass: #ButtonModel
	instanceVariableNames: 'actionHolder labelHolder stateHolder enabledHolder actionPerformedHolder helpHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!ButtonModel commentStamp: '<historical>' prior: 0!
A ButtonComposableModel is an applicative model which handle a basic button.

self example!

!ButtonModel class methodsFor: 'example' stamp: 'BenjaminVanRyseghem 4/24/2012 03:09'!
example

	| b |
	b:= ButtonModel new.
	b openWithSpec.
	b label: 'Click me'.! !

!ButtonModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 4/24/2012 02:49'!
internSpec
	<spec>
	^ {#ButtonSpec.
	    		#on:getState:action:label:menu:. 	#model. #state. #action. #label. #nil.
			#getEnabledSelector:. 				#enabled.
			#hResizing:. 						#spaceFill.
			#vResizing:. 						#spaceFill.
			#setBalloonText:.					{ #model . #help}.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!ButtonModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Button'! !

!ButtonModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 17:57'!
action

	self widget ifNotNil: [:m | m takeKeyboardFocus ].
	self actionHolder contents value.
	" Here I set a dummy value just to make the holder raise an event "
	self actionPerformedHolder contents: nil.! !

!ButtonModel methodsFor: 'protocol' stamp: ''!
action: aBlock

	self actionHolder contents: aBlock! !

!ButtonModel methodsFor: 'accessing' stamp: ''!
actionHolder

	^ actionHolder! !

!ButtonModel methodsFor: 'accessing' stamp: ''!
actionPerformedHolder
" I am a dummy holder used to announce that my action has been performed "
	^ actionPerformedHolder! !

!ButtonModel methodsFor: 'morphic' stamp: ''!
enabled

	^ self enabledHolder contents! !

!ButtonModel methodsFor: 'protocol' stamp: ''!
enabled: aBoolean
	
	^ self enabledHolder contents: aBoolean! !

!ButtonModel methodsFor: 'accessing' stamp: ''!
enabledHolder

	^ enabledHolder! !

!ButtonModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<right>'! !

!ButtonModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!ButtonModel methodsFor: 'accessing' stamp: 'StephaneDucasse 4/19/2012 17:50'!
help
	
	^ helpHolder contents! !

!ButtonModel methodsFor: 'morphic' stamp: 'StephaneDucasse 4/19/2012 17:50'!
help: aString

	helpHolder contents: aString! !

!ButtonModel methodsFor: 'initialization' stamp: 'StephaneDucasse 5/17/2012 17:50'!
initialize
	"Initialization code for ButtonComposableModel"

	super initialize.

	actionHolder := [] asValueHolder.
	labelHolder := '' asValueHolder.
	stateHolder := true asValueHolder.
	enabledHolder := true asValueHolder.
	helpHolder := nil asValueHolder.
	actionPerformedHolder := nil asValueHolder.
	
	labelHolder whenChangedDo: [	self changed: #label ].
	stateHolder whenChangedDo: [	self changed: #state ].
	enabledHolder whenChangedDo: [	self changed: #enabled ].
	helpHolder whenChangedDo: [:string | 
		widget ifNotNil: [ widget setBalloonText: string ]].! !

!ButtonModel methodsFor: 'focus' stamp: ''!
keyStroke: anEvent fromMorph: aMorph

	anEvent keyString = '<space>'
		ifTrue: [ self action ].
		
	super keyStroke: anEvent fromMorph: aMorph! !

!ButtonModel methodsFor: 'morphic' stamp: ''!
label

	^ self labelHolder contents! !

!ButtonModel methodsFor: 'protocol' stamp: ''!
label: aString

	self labelHolder contents: aString! !

!ButtonModel methodsFor: 'accessing' stamp: ''!
labelHolder

	^ labelHolder! !

!ButtonModel methodsFor: 'morphic' stamp: ''!
state

	^ self stateHolder contents! !

!ButtonModel methodsFor: 'protocol' stamp: ''!
state: aBoolean
	
	^ self stateHolder contents: aBoolean! !

!ButtonModel methodsFor: 'accessing' stamp: ''!
stateHolder

	^ stateHolder! !

!ButtonModel methodsFor: 'protocol' stamp: ''!
whenActionPerformedDo: aBlock

	self actionPerformedHolder whenChangedDo: aBlock! !

ComposableModel subclass: #CheckBoxModel
	instanceVariableNames: 'actionWhenActivatedHolder actionWhenDesactivatedHolder enabledHolder stateHolder labelClickableHolder labelHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!CheckBoxModel commentStamp: '<historical>' prior: 0!
A CheckboxModel is a spec model for Checkbox

You can also have a look at CheckBoxExample for a full example of how to use them with a group.
!

!CheckBoxModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 4/24/2012 03:10'!
internSpec
	<spec>
	^ {#CheckboxSpec.
	    		#on:selected:changeSelected:. 	#model. #state. #state:.
			#label:.								{	#model. #label		}.
			#labelClickable:.					{	#model. #labelClickable}.
			#beCheckbox.
			#hResizing:. 						#shrinkWrap.
			#vResizing:. 						#shrinkWrap.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!CheckBoxModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Radio Button'! !

!CheckBoxModel methodsFor: 'accessing' stamp: ''!
actionWhenActivatedHolder
	
	^ actionWhenActivatedHolder! !

!CheckBoxModel methodsFor: 'accessing' stamp: ''!
actionWhenDesactivatedHolder
	
	^ actionWhenDesactivatedHolder! !

!CheckBoxModel methodsFor: 'protocol' stamp: ''!
activationAction: aBlock

	"This method is used to set the action to perform when I am activated"
	self actionWhenActivatedHolder contents: aBlock! !

!CheckBoxModel methodsFor: 'protocol-events' stamp: ''!
click

	self toggleState! !

!CheckBoxModel methodsFor: 'protocol' stamp: ''!
desactivationAction: aBlock

	"This method is used to set the action to perform when I am desactivated"
	self actionWhenDesactivatedHolder contents: aBlock! !

!CheckBoxModel methodsFor: 'protocol' stamp: ''!
enabled: aBoolean

	self enabledHolder contents: aBoolean! !

!CheckBoxModel methodsFor: 'accessing' stamp: ''!
enabledHolder
	
	^ enabledHolder! !

!CheckBoxModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<right>'! !

!CheckBoxModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!CheckBoxModel methodsFor: 'initialization' stamp: 'StephaneDucasse 5/17/2012 17:50'!
initialize
	"Initialization code for RadioButton"

	super initialize.
	
	actionWhenActivatedHolder := [] asValueHolder.
	actionWhenDesactivatedHolder := [] asValueHolder.
	enabledHolder := true asValueHolder.
	stateHolder := false asValueHolder.
	labelClickableHolder := true asValueHolder. 
	labelHolder := '' asValueHolder.
	stateHolder whenChangedDo: [:bool |
		bool
			ifTrue: actionWhenActivatedHolder contents
			ifFalse: actionWhenDesactivatedHolder contents.
		self changed: #state ].
	labelClickableHolder whenChangedDo: [:aBoolean |
		widget ifNotNil: [ widget labelClickable: aBoolean ]].
	labelHolder whenChangedDo: [:label |
		widget ifNotNil: [ widget label: label ]].
	! !

!CheckBoxModel methodsFor: 'accessing' stamp: ''!
label
	
	^ labelHolder contents! !

!CheckBoxModel methodsFor: 'protocol' stamp: ''!
label: aString

	labelHolder contents: aString.! !

!CheckBoxModel methodsFor: 'protocol' stamp: ''!
labelClickable

	^ labelClickableHolder contents! !

!CheckBoxModel methodsFor: 'protocol' stamp: ''!
labelClickable: aBoolean

	labelClickableHolder contents: aBoolean! !

!CheckBoxModel methodsFor: 'morphic' stamp: ''!
state

	^ self stateHolder contents! !

!CheckBoxModel methodsFor: 'protocol' stamp: ''!
state: aBoolean

	self stateHolder contents: aBoolean! !

!CheckBoxModel methodsFor: 'protocol' stamp: ''!
stateChanged

	self changed: #state! !

!CheckBoxModel methodsFor: 'accessing' stamp: ''!
stateHolder
	
	^ stateHolder! !

!CheckBoxModel methodsFor: 'protocol' stamp: ''!
toggleState

	self state: self state not! !

!CheckBoxModel methodsFor: 'protocol-events' stamp: ''!
whenActivatedDo: aBlock

	"This method is used to propagate the event that I have been activated" 
	self stateHolder whenChangedDo: [:bool | bool ifTrue: aBlock ]! !

!CheckBoxModel methodsFor: 'protocol-events' stamp: ''!
whenDesactivatedDo: aBlock

	"This method is used to propagate the event that I have been desactivated"
	self stateHolder whenChangedDo: [:bool | bool ifFalse: aBlock ]! !

ComposableModel subclass: #DropListModel
	instanceVariableNames: 'listHolder selectionHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!DropListModel commentStamp: '<historical>' prior: 0!
A DropListModel is a Spec model for drop lists.

I am assume there is a little problem on an empty list, but frankly, who creates an empty drop list ?

See DropListExample for an example!

!DropListModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 4/24/2012 03:14'!
internSpec
	<spec>
	
	^ {#DropListSpec.
	    		#on:list:selected:changeSelected:. 	#model. #getList. #getIndex. #setIndex:.
			#hResizing:. 						#spaceFill.
			#vResizing:. 						#spaceFill.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!DropListModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Drop List'! !

!DropListModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<right>'! !

!DropListModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!DropListModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 18:12'!
getIndex

	^ selectionHolder index contents! !

!DropListModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 18:02'!
getList

	^ listHolder contents collect: #label! !

!DropListModel methodsFor: 'initialization' stamp: 'StephaneDucasse 5/17/2012 19:32'!
initialize
	"Initialization code for DropListModel"

	super initialize.

	listHolder := {} asValueHolder.
	selectionHolder := SelectionValueHolder new.
	
	listHolder whenChangedDo: [ 
					self changed: #getList.
					self internalResetSelection. ].
				
	selectionHolder whenChangedDo: [ self changed: #getIndex ].
	self whenSelectedItemChanged: [:item | item value ]! !

!DropListModel methodsFor: 'private' stamp: 'StephaneDucasse 5/17/2012 18:05'!
internalResetSelection

	selectionHolder reset.
	self getList isEmpty not
		ifTrue: [ self setSelectedIndex: 1 ]! !

!DropListModel methodsFor: 'protocol' stamp: ''!
items: aList

	listHolder contents: aList! !

!DropListModel methodsFor: 'accessing' stamp: ''!
listHolder
	
	^ listHolder! !

!DropListModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 18:05'!
resetSelection

	selectionHolder reset! !

!DropListModel methodsFor: 'protocol' stamp: ''!
selectedIndex
	"Useless method but it denies a better and more consistent API"

	^ self getIndex! !

!DropListModel methodsFor: 'private' stamp: 'StephaneDucasse 5/17/2012 18:04'!
selectedIndexHolder

	^ selectionHolder index! !

!DropListModel methodsFor: 'protocol' stamp: ''!
selectedItem

	^ self selectedItemHolder contents! !

!DropListModel methodsFor: 'private' stamp: 'StephaneDucasse 5/17/2012 18:04'!
selectedItemHolder

	^ selectionHolder selection! !

!DropListModel methodsFor: 'accessing' stamp: ''!
selectionHolder
	
	^ selectionHolder! !

!DropListModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 18:04'!
setIndex: anIndex
	| item |
	
	selectionHolder index contents: anIndex.
	item := (listHolder contents at: anIndex ifAbsent: [ nil ]).
	item value.
	selectionHolder selection contents: item.
	self changed: #getIndex! !

!DropListModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 18:02'!
setSelectedIndex: anIndex

	| selection |
	selection := listHolder contents at: anIndex ifAbsent: [ ^ self ].
	selection value.
	selectionHolder index contents: anIndex.
	selectionHolder selection contents: selection! !

!DropListModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 18:02'!
setSelectedItem: anItem
	| index |
	
	index := self getList indexOf: anItem ifAbsent: [ ^ self ].
	(listHolder contents at: index ifAbsent: [ [ ] ]) value.
	selectionHolder index contents: index.
	selectionHolder selection contents: anItem! !

!DropListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:32'!
whenSelectedItemChanged: aBlock

	selectionHolder selection whenChangedDo: aBlock! !

!DropListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:32'!
whenSelectionChanged: aBlock

	selectionHolder whenChangedDo: aBlock.! !

!DropListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:32'!
whenSelectionIndexChanged: aBlock

	selectionHolder index whenChangedDo: aBlock! !

ComposableModel subclass: #LabelModel
	instanceVariableNames: 'textHolder enabledHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!LabelModel commentStamp: '<historical>' prior: 0!
A LabelModel is a spec model for Labels!

!LabelModel class methodsFor: 'as yet unclassified' stamp: 'BenjaminVanRyseghem 4/24/2012 03:14'!
internSpec
	<spec>
	^ {#LabelSpec.
			#model:. 				#model.
			#getEnabledSelector:.	#enabled.
			#getTextSelector:.		#getText.
			#vResizing:.			#shrinkWrap.
			#hResizing:.			#spaceFill.
			#eventHandler:.	{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!LabelModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Label Morph'! !

!LabelModel methodsFor: 'protocol' stamp: ''!
disable

	self enabled: false! !

!LabelModel methodsFor: 'protocol' stamp: ''!
enable

	self enabled: true! !

!LabelModel methodsFor: 'morphic' stamp: ''!
enabled

	^ enabledHolder contents! !

!LabelModel methodsFor: 'protocol' stamp: ''!
enabled: aBoolean

	enabledHolder contents: aBoolean! !

!LabelModel methodsFor: 'morphic' stamp: ''!
getText

	^ textHolder contents! !

!LabelModel methodsFor: 'initialization' stamp: ''!
initialize
	"Initialization code for LabelModel"

	super initialize.

	textHolder := '' asValueHolder.
	enabledHolder := true asValueHolder.
	textHolder whenChangedDo: [ self changed: #getText ].
	enabledHolder whenChangedDo: [ self changed: #enabled ].! !

!LabelModel methodsFor: 'protocol' stamp: ''!
text: aText

	textHolder contents: aText! !

ComposableModel subclass: #ListComposableModel
	instanceVariableNames: 'selectionHolder listHolder menuHolder shortcutsHolder multiSelectionHolder multiSelection allowToSelect listItemsCache filteringBlockHolder sortingBlockHolder wrapBlockHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!ListComposableModel commentStamp: '<historical>' prior: 0!
A ListComposableModel is an applicative model which handle a basic list.

| t |
t:= ListComposableModel new.
t openWithSpec.
t items: (Smalltalk allClasses).


self example

| t |
t:= ListComposableModel new.
t openWithSpec.
t sortingBlock: [:a :b| a name > b name].
t items: (Smalltalk allClasses).


| t |
t:= ListComposableModel new.
t openWithSpec.
t filteringBlock: [:col | col select: [:each | each name beginsWith: 'Zn']].
t sortingBlock: [:a :b| a name > b name].
t items: (Smalltalk allClasses).
!

ListComposableModel subclass: #IconListModel
	instanceVariableNames: 'iconHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!IconListModel commentStamp: '<historical>' prior: 0!
An IconListModel is a spec model for icon list!

!IconListModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 4/24/2012 03:15'!
internSpec
	<spec>
	^ {#IconListSpec.
	    		#model:. 						#model.
			#getListSizeSelector:. 			#listSize.
			#getListElementSelector:. 		#listElementAt:.
			#getIndexSelector:. 			#getIndex.
			#setIndexSelector:. 			#setIndex:.
			#getSelectionListSelector:.		#getSelectionStateFor:.
			#setSelectionListSelector:.		#setSelectionStateFor:at:.
			#getIconSelector:.				#getIconFor:.
			#resetListSelector:.				#resetListSelection.
			#getMenuSelector:.			#menu:shifted:.
			#keystrokeSelector:.			#keyStroke:fromMorph:.
			#setMultipleSelection:.			{#model. #multiSelection}.
			#wrapSelector:. 				#wrapItem:.
			#hResizing:. 					#spaceFill.
			#vResizing:. 					#spaceFill}! !

!IconListModel methodsFor: 'morphic' stamp: ''!
getIconFor: anItem

	^ iconHolder contents cull: anItem cull: self! !

!IconListModel methodsFor: 'protocol' stamp: ''!
icons: aBlock

	iconHolder contents: aBlock! !

!IconListModel methodsFor: 'initialization' stamp: ''!
initialize
	"Initialization code for IconListModel"

	super initialize.

	iconHolder := [:item | nil ] asValueHolder.! !

!ListComposableModel class methodsFor: 'example' stamp: 'StephaneDucasse 5/17/2012 18:02'!
example
	| instance |

	instance := self new.
	instance openWithSpec.
	instance items: (Smalltalk allClasses).! !

!ListComposableModel class methodsFor: 'specs' stamp: 'StephaneDucasse 5/17/2012 18:15'!
internSpec
	<spec>
	^ {#ListSpec.
	    		#model:. 					#model.
			#getListSizeSelector:. 			#listSize.
			#getIndexSelector:. 			#getIndex.
			#setIndexSelector:. 			#setIndex:.
			#getSelectionListSelector:.		#getSelectionStateFor:.
			#setSelectionListSelector:.		#setSelectionStateFor:at:.
			#getListElementSelector:. 		#listElementAt:.
			#resetListSelector:.			#resetListSelection.
			#getMenuSelector:.			#menu:shifted:.
			#keystrokeSelector:.			#keyStroke:fromMorph:.
			#setMultipleSelection:.			{#model. #multiSelection}.
			#wrapSelector:. 				#wrapItem:.
			#hResizing:. 					#spaceFill.
			#vResizing:. 					#spaceFill}! !

!ListComposableModel class methodsFor: 'specs' stamp: ''!
title

	^ 'List'! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:10'!
allowToSelect
	"Return whether the list items can be selected or not"

	^ allowToSelect contents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:03'!
allowToSelect: aBoolean
	"Set if the list items can be selected or not"

	allowToSelect contents: aBoolean! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:03'!
beMultipleSelection
	"Make list selection multiple"

	self multiSelection: true! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:04'!
beSingleSelection
	"Make list selection single"

	self multiSelection: false! !

!ListComposableModel methodsFor: 'initialize' stamp: 'SD 4/19/2012 15:26'!
defaultFilteringBlock 
	^ [:col | col]! !

!ListComposableModel methodsFor: 'initialize' stamp: 'StephaneDucasse 4/17/2012 19:15'!
defaultSortingBlock 
	^ [:a :b | true]! !

!ListComposableModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"	
	^ '<right>'! !

!ListComposableModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 18:12'!
filteringBlock: aBlock
	"To set the filtering of the items"
	
	filteringBlockHolder contents: aBlock! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:22'!
getIndex
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Return the index of the selected item"
	
	^  selectionHolder index contents! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:24'!
getSelectionStateFor: anIndex
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Return the current state of the item -if selected or not - iun a multiple selection list"
	"Answer true if the item at index _anIndex_ is selected"
	
	^ (multiSelectionHolder at: anIndex ifAbsent: [ ^ false ]) == true! !

!ListComposableModel methodsFor: 'initialize' stamp: 'StephaneDucasse 5/17/2012 18:57'!
initialize

	super initialize.

	selectionHolder := SelectionValueHolder new.
	listHolder := Array new asValueHolder.
	sortingBlockHolder := self defaultSortingBlock asValueHolder.
	filteringBlockHolder := self defaultFilteringBlock asValueHolder.
	wrapBlockHolder := [ :object | object asStringOrText ] asValueHolder.
	menuHolder := [ :menu :shifted | nil ] asValueHolder.
	shortcutsHolder := [ :event :sourceMorph |  ] asValueHolder.
	multiSelectionHolder := IdentityDictionary new asValueHolder.
	multiSelection := false asValueHolder.
	allowToSelect := true asValueHolder.

	self registerEvents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:05'!
items: aList
	"Set the items of the list"
	"aList is a collection of your domain specific items"

	listHolder contents: (aList sorted: sortingBlockHolder contents)! !

!ListComposableModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:20'!
keyStroke: anEvent fromMorph: aMorph

	"Simulate a click on the selected item when space is pressed"
	anEvent keyString = String space
		ifTrue: [| item |
			item := self selectedItem. 
			(item notNil and: [ item respondsTo: #click ])
				ifTrue: [ ^ item click ]].

	"Now check the shortcuts defined by the user"
	shortcutsHolder contents cull: anEvent cull: aMorph.

	"Finally do a super call for the focus change"
	^ super keyStroke: anEvent fromMorph: aMorph! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:25'!
listElementAt: anIndex
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Return the item at index _anIndex_"
	
	^ self listItems at: anIndex ifAbsent: [ nil ]! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:07'!
listItems
	"Return the items of the list. They are your domain specific items"
	
	^ listItemsCache ifNil: [ listItemsCache :=  filteringBlockHolder contents value: listHolder contents ]
	! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:48'!
listSize
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Return the size of the list"
	
	^ self listItems size! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:07'!
menu: aBlock
	"Set the block used to defined the menu"
	menuHolder contents: aBlock! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:49'!
menu: aMenu shifted: aBoolean
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Build the menu when you right click on an item"

	^ menuHolder contents cull: aMenu cull: aBoolean! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:10'!
multiSelection
	"Return true if the list has a multiple selection"
	"False if the list has a single selection"
	
	^ multiSelection contents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:11'!
multiSelection: aBoolean
	"Make the list seelction become multiple if aBoolean is true"
	"Otherwise set the selection as single"
	
	multiSelection contents: aBoolean.
	! !

!ListComposableModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 5/7/2012 13:23'!
refreshListItems

	listItemsCache := nil.
	^ self changed: #listElementAt:! !

!ListComposableModel methodsFor: 'initialize' stamp: 'StephaneDucasse 5/17/2012 18:17'!
registerEvents

	listHolder whenChangedDo: [ self refreshListItems ].
	filteringBlockHolder whenChangedDo: [ self refreshListItems ].
	selectionHolder index whenChangedDo: [ self changed: #getIndex ].
	multiSelection whenChangedDo: [ :b | widget ifNotNil: [ widget setMultipleSelection: b ] ].
	
	sortingBlockHolder
		whenChangedDo: [ 
			listHolder contents: (listHolder contents sorted: sortingBlockHolder contents).
			self changed: #listElementAt: ].
! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:12'!
resetFilteringBlock
	"Reset the filtering block with the default value which consists in showing everything"

	filteringBlockHolder contents: self defaultFilteringBlock! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:50'!
resetListSelection
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Reset the selection manager for multiple selection lists"
	
	multiSelectionHolder removeAll! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:12'!
resetSelection
	"Unselect every items"

	selectionHolder reset.
	multiSelectionHolder contents removeAll! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:13'!
resetSortingBlock
	"Reset the sortering block with the default value which consists in not sorting"

	sortingBlockHolder contents: self defaultSortingBlock! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:16'!
selectedIndex
	"Return the index of the selected item"
	"In the case of a multiple selection list, it returns the last selected item"

	^ self getIndex! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:15'!
selectedIndexes
	"Return the indexes of selected items on the case of a multiple selection list"
	
	^ multiSelectionHolder keys select: [:index | multiSelectionHolder at: index ]! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:15'!
selectedItem
	"Return the selected item"
	"In the case of a multiple selection list, it returns the last selected item"
	
	^ self selectedItemHolder contents! !

!ListComposableModel methodsFor: 'private' stamp: 'StephaneDucasse 5/17/2012 18:04'!
selectedItemHolder

	^ selectionHolder selection! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:18'!
selectedItems
	"Return all the selected items in the case of a multiple selection list"

	^ multiSelectionHolder keys select: [:index | multiSelectionHolder at: index ] thenCollect: [:index | self listElementAt: index ]! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 19:18'!
setIndex: anIndex
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Set the index of the selected item when you click on an item"

	self allowToSelect ifFalse: [ ^ self ].
	self okToChange ifFalse: [ ^ self ].
	selectionHolder index contents: anIndex.
	selectionHolder selection contents: (self listElementAt: anIndex).! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:19'!
setSelectedIndex: anIndex
	"Set the index of the item you want to be selected"
	| idx selection |
	
	self allowToSelect ifFalse: [ ^ self ].
	self okToChange ifFalse: [ ^ self ].
	idx := anIndex min: self listSize.
	selection := self listElementAt: idx.
	selection ifNil: [ ^ self ].
	selectionHolder index contents: idx.
	selectionHolder selection contents: selection.! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:20'!
setSelectedItem: anItem
	"Set the item you want to be selected"
	| index |

	self okToChange ifFalse: [ ^ self ].
	index := self listItems identityIndexOf: anItem ifAbsent: [ ^ self ].
	selectionHolder index contents: index.
	selectionHolder selection contents: anItem! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:55'!
setSelectionStateFor: anIndex at: aBoolean
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Set the selection state of the item at index _anIndex_ in the case of multiple selection list"

	self allowToSelect ifFalse: [ ^ self ].
	^ multiSelectionHolder at: anIndex put: aBoolean! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:20'!
shortcuts: aBlock
	"Set the block used to defined the shortcuts"
	
	shortcutsHolder contents: aBlock! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 18:13'!
sortingBlock: aBlock
	"To set the ordering of the items"
	
	sortingBlockHolder contents: aBlock! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:20'!
updateList
	"Refresh the list"
	
	self changed: #listElementAt:.
	self resetSelection! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:31'!
whenListChanged: aBlock
	"Specify a block to value after the contents of the list has changed"
	"Basically when you set a new list of items"

	listHolder whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:33'!
whenSelectedItemChanged: aBlock
	"Set a block to value when the select item is changed"
	
	selectionHolder selection whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:34'!
whenSelectionChanged: aBlock
	"Set a block to value when the selection of the list has changed"

	selectionHolder whenChangedDo: aBlock.
	multiSelectionHolder whenChangedDo: aBlock.! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:36'!
whenSelectionIndexChanged: aBlock
	"Set a block to value when the selection index has changed"
	
	selectionHolder index whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:57'!
wrapItem: anObject
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Return the item _anObject_ wrapped"

	^ wrapBlockHolder contents value: anObject! !

!ListComposableModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 19:23'!
wrappingBlock: aBlock
	"Set the one argument block used to wrap your domain specific items"
	"The block should return something that can be displayed in a list - like a String or a Text"
	wrapBlockHolder contents: aBlock! !

ListComposableModel subclass: #MultiColumnListModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!MultiColumnListModel commentStamp: '<historical>' prior: 0!
A MultiColumnListModel is a spec model for Multi columns list!

!MultiColumnListModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 5/7/2012 16:06'!
internSpec
	<spec>
	^ {#MultiColumnListSpec.
			#wrapSelector:. 				#wrapItem:.
	    		#model:. 						#model.
			#getListSizeSelector:. 			#listSize.
			#getIndexSelector:. 			#getIndex.
			#setIndexSelector:. 			#setIndex:.
			#getSelectionListSelector:.		#getSelectionStateFor:.
			#setSelectionListSelector:.		#setSelectionStateFor:at:.
			#getListElementSelector:. 		#listElementAt:.
			#resetListSelector:.				#resetListSelection.
			#getMenuSelector:.			#menu:shifted:.
			#keystrokeSelector:.			#keyStroke:fromMorph:.
			#setMultipleSelection:.			{#model. #multiSelection}.
			#hResizing:. 					#spaceFill.
			#vResizing:. 					#spaceFill}! !

!MultiColumnListModel methodsFor: 'remove me' stamp: 'StephaneDucasse 5/17/2012 18:03'!
getIndex

	^ selectionHolder index contents! !

!MultiColumnListModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 5/7/2012 13:20'!
listElementAt: anIndex

	^ self wrapItem: (self listItems at: anIndex)! !

!MultiColumnListModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 18:03'!
setIndex: anIndex

	self allowToSelect ifFalse: [ ^ self ].
	self okToChange ifFalse: [ ^ self ].
	selectionHolder index contents: anIndex.
	selectionHolder selection contents: (self listItems at: anIndex ifAbsent: [ nil ]).! !

!MultiColumnListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 5/7/2012 15:57'!
setSelectedIndex: anIndex
	| idx selection |
	
	self allowToSelect ifFalse: [ ^ self ].
	self okToChange ifFalse: [ ^ self ].
	idx := anIndex min: self listSize.
	selection := (self listItems at: idx).
	selection ifNil: [ ^ self ].
	
	selectionHolder index contents: idx.
	selectionHolder selection contents: selection.! !

ComposableModel subclass: #MethodToolbar
	instanceVariableNames: 'methodHolder browseModel sendersModel implementorsModel versionModel dropListModel model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-PolyWidgets'!

!MethodToolbar commentStamp: '<historical>' prior: 0!
A MethodToolbar is xxxxxxxxx.

b := MethodToolbar new.
b openWithSpec.
b method: (ButtonModel>>#state:).
b method: nil!

!MethodToolbar class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 3/23/2012 16:56'!
internSpec
	<spec>
	^ { #Panel.
			#changeTableLayout.
			#listDirection:.		#rightToLeft.
			#addMorph:. {#model.	#browseModel.	#internSpec}.
			#addMorph:. {#model.	#sendersModel.	#internSpec}.
			#addMorph:. {#model.	#implementorsModel.	#internSpec}.
			#addMorph:. {#model.	#versionModel.	#internSpec}.
			#addMorph:. {#model.	#dropListModel.	#internSpec}.
			#hResizing:.	#spaceFill.
			#vResizing:.	#shrinkWrap.	}! !

!MethodToolbar class methodsFor: 'specs' stamp: ''!
title

	^ 'Toolbar'! !

!MethodToolbar methodsFor: 'private' stamp: ''!
addAll: aWindow  withSpec: aSpec

	aWindow
		addMorph: (self buildWithSpec: aSpec)
		fullFrame: (LayoutFrame 
			fractions: (0@0 corner: 1@0) 
			offsets: (0@0 corner: 0@30)).! !

!MethodToolbar methodsFor: 'private' stamp: ''!
browseMethod

	methodHolder contents
		ifNotNil: [:aMethod | aMethod browse ]! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
browseModel

	^ browseModel! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
dropListModel

	^ dropListModel! !

!MethodToolbar methodsFor: 'private' stamp: ''!
implementorsMethod

	methodHolder contents
		ifNotNil: [:aMethod | self model browseMessagesFrom: aMethod selector ]! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
implementorsModel

	^ implementorsModel! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
initialize
	"Initialization code for MethodToolbar"

	super initialize.

	methodHolder := nil asValueHolder.
	browseModel := self instantiate: ButtonModel.
	sendersModel := self instantiate: ButtonModel.
	implementorsModel := self instantiate: ButtonModel.
	versionModel := self instantiate: ButtonModel.
	dropListModel := self instantiate: DropListModel.
	model := AbstractTool new.

	self setFocusOrder.
	
	self setBrowseModel.
	self setVersionModel.
	self setSendersModel.
	self setImplementorsModel.

	"self registerForChanges."! !

!MethodToolbar methodsFor: 'protocol' stamp: ''!
method: aMethod

	methodHolder contents: aMethod! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
model

	^ model! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
registerForChanges

	methodHolder whenChangedDo: [:contents || boolean |
		boolean := contents
						ifNil: [ false ]
						ifNotNil: [ true ].
		browseModel state: boolean.
		sendersModel state: boolean.
		versionModel state: boolean.
		implementorsModel state: boolean]! !

!MethodToolbar methodsFor: 'private' stamp: ''!
sendersMethod

	methodHolder contents
		ifNotNil: [:aMethod | self model browseSendersOfMessagesFrom: aMethod selector ]! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
sendersModel

	^ sendersModel! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
setBrowseModel
	
	browseModel
		state: false;
		label: 'Browse';
		action: [ self browseMethod ].
	
! !

!MethodToolbar methodsFor: 'protocol' stamp: ''!
setDropListItems: aCollection

	dropListModel items: aCollection! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
setFocusOrder

	self focusOrder
		add: browseModel;
		add: sendersModel;
		add: implementorsModel;
		add: versionModel.
		! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
setImplementorsModel
	
	implementorsModel
		state: false;
		label: 'Implementors';
		action: [ self implementorsMethod ].
	
! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
setSendersModel
	
	sendersModel
		state: false;
		label: 'Senders';
		action: [ self sendersMethod ].
	
! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
setVersionModel
	
	versionModel
		state: false;
		label: 'Version';
		action: [ self versionMethod ].
	
! !

!MethodToolbar methodsFor: 'private' stamp: ''!
versionMethod

	methodHolder contents
		ifNotNil: [:aMethod | self model browseVersionsFrom: aMethod ]! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
versionModel

	^ versionModel! !

ComposableModel subclass: #OkToolbar
	instanceVariableNames: 'okButton okAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-PolyWidgets'!

!OkToolbar commentStamp: '<historical>' prior: 0!
An OkCancelToolbar is a model for a basic Ok toolbar!

OkToolbar subclass: #OkCancelToolbar
	instanceVariableNames: 'cancelButton cancelAction cancelled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-PolyWidgets'!

!OkCancelToolbar commentStamp: '<historical>' prior: 0!
An OkCancelToolbar is a model for a basic Ok-Cancel toolbar!

!OkCancelToolbar class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 3/29/2012 10:20'!
internSpec
	<spec>
	^ { #Panel.
			#changeTableLayout.
			#listDirection:.		#rightToLeft.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#okButton.		#internSpec.}.
			#add:. {#model.	#cancelButton.	#internSpec.}.
			#hSpaceFill.
			#vShrinkWrap.	}! !

!OkCancelToolbar class methodsFor: 'specs' stamp: ''!
title

	^ 'Ok Cancel'! !

!OkCancelToolbar methodsFor: 'protocol' stamp: ''!
cancelAction

	^ cancelAction contents! !

!OkCancelToolbar methodsFor: 'protocol' stamp: ''!
cancelAction: aBlock

	^ cancelAction contents: aBlock! !

!OkCancelToolbar methodsFor: 'accessing' stamp: ''!
cancelButton

	^ cancelButton! !

!OkCancelToolbar methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 5/11/2012 18:32'!
cancelled

	^ cancelled contents! !

!OkCancelToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 5/11/2012 18:31'!
initialize
	"Initialization code for OkCancelToolbar"

	super initialize.

	cancelButton := self instantiate: ButtonModel.
	cancelled := false asValueHolder.
	
	cancelAction := [ true ] asValueHolder.
	
	self registerCancelButtonEvents.
	self focusOrder
		add: cancelButton! !

!OkCancelToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 5/11/2012 18:31'!
registerCancelButtonEvents

	cancelButton 
		label: 'Cancel';
		enabled: true;
		state: false;
		action: [
			cancelAction contents value == false
					ifFalse: [
						cancelled contents: true.
						owner 
							ifNil: [ self delete ]
							ifNotNil: [ owner delete ]]].! !

!OkToolbar class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 5/10/2012 13:30'!
internSpec
	<spec>
	^ { #Panel.
			#changeTableLayout.
			#listDirection:.		#rightToLeft.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#okButton.		#internSpec.}.
			#hSpaceFill.
			#vShrinkWrap.	}! !

!OkToolbar class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 5/10/2012 13:07'!
title

	^ 'Ok Toolbar'! !

!OkToolbar methodsFor: 'private' stamp: 'BenjaminVanRyseghem 5/10/2012 13:05'!
addAll: aWindow  withSpec: aSpec

	aWindow
		addMorph: (self buildWithSpec: aSpec)
		fullFrame: (LayoutFrame 
			fractions: (0@0 corner: 1@0) 
			offsets: (0@0 corner: 0@30)).! !

!OkToolbar methodsFor: 'private' stamp: 'BenjaminVanRyseghem 5/10/2012 13:05'!
hFill

	^ Morph new
		color: Color transparent;
		height: 0;
		vResizing: #rigid;
		hResizing: #spaceFill! !

!OkToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 5/10/2012 13:09'!
initialize
	"Initialization code for OkCancelToolbar"

	super initialize.

	okButton := self instantiate: ButtonModel.
	
	okAction := [ true ] asValueHolder.
	
	self registerOkButtonEvents.
	self focusOrder
		add: okButton! !

!OkToolbar methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 5/10/2012 13:05'!
okAction

	^ okAction contents! !

!OkToolbar methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 5/10/2012 13:05'!
okAction: aBlock

	^ okAction contents: aBlock! !

!OkToolbar methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 5/10/2012 13:05'!
okButton

	^ okButton! !

!OkToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 5/11/2012 16:16'!
registerOkButtonEvents

	okButton 
		label: 'Ok';
		enabled: true;
		state: false;
		action: [ 
				okAction contents value == false
					ifFalse: [
						owner 
							ifNil: [ self delete ]
							ifNotNil: [ owner delete ]]].! !

ComposableModel subclass: #PickListModel
	instanceVariableNames: 'defaultValueHolder labelClickableHolder listModel pickedItemsHolder wrapHolder associationsItemToWrapper associationsWrapperToItem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-PolyWidgets'!

!PickListModel commentStamp: '<historical>' prior: 0!
A PickList is a tick list done using spec.!

!PickListModel class methodsFor: 'example' stamp: ''!
example
	"self example"
	| instance |
	
	instance := self new.
	instance openWithSpec.
	instance items: { {1. 2. 4}. 'Foo'. 123}! !

!PickListModel class methodsFor: 'specs' stamp: ''!
internSpec
	<spec>
	^{#model. #listModel. #internSpec}! !

!PickListModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Pick List Example'! !

!PickListModel methodsFor: 'private' stamp: ''!
addPicked: item

	pickedItemsHolder contents add: item.
	pickedItemsHolder contentsChanged: true to: item.! !

!PickListModel methodsFor: 'protocol' stamp: ''!
checkboxFor: anItem

	^ associationsItemToWrapper at: anItem ifAbsent: [ nil ]! !

!PickListModel methodsFor: 'protocol' stamp: ''!
defaultValue

	^ defaultValueHolder contents! !

!PickListModel methodsFor: 'protocol' stamp: ''!
defaultValue: aBoolean

	defaultValueHolder contents: aBoolean! !

!PickListModel methodsFor: 'initialization' stamp: 'StephaneDucasse 5/17/2012 19:31'!
initialize
	"Initialization code for PickList"

	super initialize.

	listModel := self instantiate: ListComposableModel.
	wrapHolder := [ :i | i printString ] asValueHolder. 
	pickedItemsHolder := Set new asValueHolder.
	associationsWrapperToItem := Dictionary new asValueHolder.
	associationsItemToWrapper := Dictionary new asValueHolder.
	labelClickableHolder := true asValueHolder.
	
	listModel whenListChanged: [ self focusOrder removeAll ].
	listModel wrappingBlock: [ :item | item ].
	labelClickableHolder whenChangedDo: [:aBoolean | 
		associationsWrapperToItem keys do: [:e | e labelClickable: aBoolean ]].
	defaultValueHolder := false asValueHolder.
	defaultValueHolder whenChangedDo: [:value | associationsWrapperToItem keysDo: [:cb | cb state: value ]].! !

!PickListModel methodsFor: 'protocol' stamp: ''!
itemFor: aCheckbox

	^ associationsWrapperToItem at: aCheckbox ifAbsent: [ nil ]! !

!PickListModel methodsFor: 'protocol' stamp: ''!
items: aCollection

	associationsWrapperToItem removeAll.
	pickedItemsHolder contents removeAll.
	pickedItemsHolder contentsChanged: false to: nil.
	listModel items: (aCollection collect: [:e | self wrap: e ]).! !

!PickListModel methodsFor: 'protocol' stamp: ''!
labelClickable
	
	^ labelClickableHolder contents! !

!PickListModel methodsFor: 'protocol' stamp: ''!
labelClickable: aBoolean

	labelClickableHolder contents: aBoolean! !

!PickListModel methodsFor: 'accessing' stamp: ''!
listModel

	^ listModel! !

!PickListModel methodsFor: 'protocol' stamp: ''!
pickedItems

	^ pickedItemsHolder contents! !

!PickListModel methodsFor: 'private' stamp: ''!
removePicked: item

	pickedItemsHolder contents remove: item ifAbsent: [ ^ self ].
	pickedItemsHolder contentsChanged: false to: item.! !

!PickListModel methodsFor: 'protocol' stamp: ''!
resetSelection

	listModel resetSelection! !

!PickListModel methodsFor: 'protocol' stamp: ''!
selectedIndex

	^ listModel selectedIndex! !

!PickListModel methodsFor: 'protocol' stamp: ''!
selectedItem

	^ associationsWrapperToItem 
		at: listModel selectedItem
		ifAbsent: [ nil ]! !

!PickListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:31'!
whenListChanged: aBlock

	listModel whenListChanged: aBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:31'!
whenListChanges: aBlock

	listModel whenListChanged: aBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: ''!
whenPickedItemsChanges: aBlock

	pickedItemsHolder whenChangedDo: aBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:32'!
whenSelectedItemChanged: aBlock
	| newBlock |
	
	newBlock := [ :item | aBlock cull: (associationsWrapperToItem at: item ifAbsent: [ nil ])].
	listModel whenSelectedItemChanged: newBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:32'!
whenSelectedItemChanges: aBlock
	| newBlock |
	
	newBlock := [ :item | aBlock cull: (associationsWrapperToItem at: item ifAbsent: [ nil ])].
	listModel whenSelectedItemChanged: newBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:32'!
whenSelectionChanged: aBlock

	listModel whenSelectionChanged: aBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:32'!
whenSelectionIndexChanged: aBlock

	listModel whenSelectionIndexChanges: aBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:32'!
whenSelectionIndexChanges: aBlock

	listModel whenSelectionIndexChanged: aBlock! !

!PickListModel methodsFor: 'private' stamp: 'StephaneDucasse 5/17/2012 18:14'!
wrap: item
	| checkBox |

	associationsItemToWrapper 
		at: item
		ifPresent: [:cb | ^ cb ].

	checkBox := self instantiate: CheckBoxModel.
	checkBox label: (wrapHolder contents cull: item cull: self).
	checkBox whenActivatedDo: [ 
		self addPicked: item.
		listModel setSelectedItem: checkBox.
		listModel takeKeyboardFocus ].
	checkBox whenDesactivatedDo: [ 
		self removePicked: item.
		listModel setSelectedItem: checkBox.
		listModel takeKeyboardFocus ].
	checkBox 
		state: self defaultValue;
		labelClickable: self labelClickable.
	associationsWrapperToItem at: checkBox put: item.
	associationsItemToWrapper at: item put: checkBox.
	^ checkBox! !

!PickListModel methodsFor: 'accessing' stamp: ''!
wrapHolder

	^ wrapHolder! !

!PickListModel methodsFor: 'protocol' stamp: ''!
wrappingBlock: aBlock

	wrapHolder contents: aBlock! !

ComposableModel subclass: #RadioButtonModel
	instanceVariableNames: 'actionWhenActivatedHolder actionWhenDesactivatedHolder enabledHolder stateHolder label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!RadioButtonModel commentStamp: '<historical>' prior: 0!
A RadioButtonModel is a spec model for RadioButton

!! The method privateSetState: is really private and should only be used by RadioButtonGroup !!

You can also have a look at RadioButtonGroupExample for a full example of how to use them with a group.!

!RadioButtonModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 4/24/2012 03:16'!
internSpec
<spec>
	^ {#CheckboxSepc.
	    		#on:selected:changeSelected:. 	#model. #state. #state:.
			#label:.								{	#model. #label		}.
			#beRadioButton.
			#hResizing:. 						#shrinkWrap.
			#vResizing:. 						#shrinkWrap.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!RadioButtonModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Radio Button'! !

!RadioButtonModel methodsFor: 'accessing' stamp: ''!
actionWhenActivatedHolder
	
	^ actionWhenActivatedHolder! !

!RadioButtonModel methodsFor: 'accessing' stamp: ''!
actionWhenDesactivatedHolder
	
	^ actionWhenDesactivatedHolder! !

!RadioButtonModel methodsFor: 'protocol' stamp: ''!
activationAction: aBlock

	"This method is used to set the action to perform when I am activated"
	self actionWhenActivatedHolder contents: aBlock! !

!RadioButtonModel methodsFor: 'protocol' stamp: ''!
desactivationAction: aBlock

	"This method is used to set the action to perform when I am desactivated"
	self actionWhenDesactivatedHolder contents: aBlock! !

!RadioButtonModel methodsFor: 'protocol' stamp: ''!
enabled: aBoolean

	self enabledHolder contents: aBoolean! !

!RadioButtonModel methodsFor: 'accessing' stamp: ''!
enabledHolder
	
	^ enabledHolder! !

!RadioButtonModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<right>'! !

!RadioButtonModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!RadioButtonModel methodsFor: 'initialization' stamp: ''!
initialize
	"Initialization code for RadioButton"

	super initialize.
	
	actionWhenActivatedHolder := [] asValueHolder.
	actionWhenDesactivatedHolder := [] asValueHolder.
	enabledHolder := true asValueHolder.
	stateHolder := true asValueHolder.
	stateHolder whenChangedDo: [:bool |
		bool
			ifTrue: actionWhenActivatedHolder contents
			ifFalse: actionWhenDesactivatedHolder contents ].! !

!RadioButtonModel methodsFor: 'accessing' stamp: ''!
label
	
	^ label! !

!RadioButtonModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 17:57'!
label: aString

	label := aString.
	self widget
		ifNotNil: [:m | m label: label ]! !

!RadioButtonModel methodsFor: 'private' stamp: ''!
privateSetState: aBoolean

	self stateHolder contents: aBoolean.
	self changed: #state! !

!RadioButtonModel methodsFor: 'morphic' stamp: ''!
state

	^ self stateHolder contents! !

!RadioButtonModel methodsFor: 'protocol' stamp: ''!
state: aBoolean

	self state
		ifFalse: [ self stateHolder contents: true ].! !

!RadioButtonModel methodsFor: 'accessing' stamp: ''!
stateHolder
	
	^ stateHolder! !

!RadioButtonModel methodsFor: 'protocol-events' stamp: ''!
whenActivatedDo: aBlock

	"This method is used to propagate the event that I have been activated" 
	self stateHolder whenChangedDo: [:bool | bool ifTrue: aBlock ]! !

!RadioButtonModel methodsFor: 'protocol-events' stamp: ''!
whenDesactivatedDo: aBlock

	"This method is used to propagate the event that I have been desactivated"
	self stateHolder whenChangedDo: [:bool | bool ifFalse: aBlock ]! !

ComposableModel subclass: #SearchableList
	instanceVariableNames: 'listModel searchModel title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-PolyWidgets'!

!SearchableList commentStamp: '<historical>' prior: 0!
SearchableList 
	new items: #( aaa abc axz bbb bfbg bof c);
	openWithSpec
	
should it be a subclass of LIstModel?
Because now I will have to delegate a lot? and I cannot plug a IconList

How do I make textModel react to every keystroke without an accept!

!SearchableList class methodsFor: 'spec' stamp: 'StephaneDucasse 5/17/2012 17:30'!
internSpec

	^ {#ComposableSpec .
		#add: . {{#model . #searchModel. #internSpec}.
				#layout: . {#FrameLayout . 
								#fractions:offsets: . (0@0 corner: 1@0). (0@0 corner: (0@25))}} .
		#add: . {{#model . #listModel. #internSpec}.
				#layout: . {#FrameLayout . 
								#fractions:offsets: . (0@0 corner: 1@1). (0@25corner:  0@0)}}
		}! !

!SearchableList methodsFor: 'initialize' stamp: 'StephaneDucasse 5/17/2012 17:37'!
initialize

	super initialize.
	listModel := self instantiate: ListComposableModel.
	searchModel := self instantiate: TextInputFieldModel.
	searchModel 
		autoAccept: true;
		ghostText: 'filter'.
	
	searchModel whenTextChanges: [:newText || text |
		text :=  searchModel getText asLowercase.
		text isEmpty
			ifTrue: [ listModel resetFilteringBlock ]
			ifFalse: [
				listModel filteringBlock: [:col | col select: [:e | e asLowercase beginsWith: searchModel getText asLowercase ]]]]! !

ComposableModel subclass: #TextModel
	instanceVariableNames: 'aboutToStyleHolder behaviorHolder textHolder actionToPerformHolder readSelectionHolder enabledHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!TextModel commentStamp: '<historical>' prior: 0!
A TextComposableModel is an applicative model which handle a basic text.

| t |
t:= TextComposableModel new.
t openWithSpec.
t textHolder contents: 'This is a test'.

self example!

TextModel subclass: #TextInputFieldModel
	instanceVariableNames: 'ghostText alwaysAccept acceptOnCR autoAccept entryCompletion'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!TextInputFieldModel commentStamp: '<historical>' prior: 0!
A TextInputFieldModel is a spec model for TextInputField.!

!TextInputFieldModel class methodsFor: 'example' stamp: ''!
example
	| instance |
	
	instance := self new.
	instance openWithSpec.
	instance ghostText: 'This is a ghost'.
	instance text: 'This is a text'! !

!TextInputFieldModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 5/9/2012 17:20'!
internSpec
	<spec>
	^ {#TextField.
			#convertTo:. 							String.
			#on:text:accept:readSelection:menu:. 	#model. #getText. #accept:. nil. nil.
			#entryCompletion:. 				{	#model. 	#entryCompletion	}.
			#autoAccept:.						{	#model. 	#autoAccept			}.	
			#acceptOnCR:. 					{	#model. 	#acceptOnCR			}.
			#ghostText:.						{	#model. 	#ghostText			}.
			#enabled:.							{	#model. 	#enabled				}.
			#hResizing:. 							#spaceFill.
			#vResizing:. 							#spaceFill.
			#hideScrollBarsIndefinitely.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!TextInputFieldModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Text'! !

!TextInputFieldModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 17:45'!
accept: aText

	textHolder contents: aText.
	actionToPerformHolder contents cull: aText.
	self changed: #clearUserEdits.! !

!TextInputFieldModel methodsFor: 'protocol' stamp: ''!
acceptOnCR

	^ acceptOnCR contents! !

!TextInputFieldModel methodsFor: 'protocol' stamp: ''!
acceptOnCR: aBoolean

	acceptOnCR contents: aBoolean! !

!TextInputFieldModel methodsFor: 'accessing' stamp: ''!
actionToPerformHolder

	^ actionToPerformHolder! !

!TextInputFieldModel methodsFor: 'protocol' stamp: ''!
autoAccept

	^ autoAccept contents! !

!TextInputFieldModel methodsFor: 'protocol' stamp: ''!
autoAccept: aBoolean

	autoAccept contents: aBoolean.! !

!TextInputFieldModel methodsFor: 'private' stamp: ''!
defaultEntryCompletion

	| applicants |
	applicants := (Array
				streamContents: [:strm | Symbol
						allSymbolTablesDo: [:each | (each notEmpty
									and: [each first canBeGlobalVarInitial
											and: [Smalltalk globals includesKey: each]])
								ifTrue: [strm nextPut: each]]]) sort.

	^ EntryCompletion new
				dataSourceBlock: [:currText | applicants];
				filterBlock: [:currApplicant :currText | currText size > 3
						and: [currApplicant asUppercase includesSubString: currText asString asUppercase]].! !

!TextInputFieldModel methodsFor: 'accessing' stamp: ''!
entryCompletion

	^ entryCompletion contents! !

!TextInputFieldModel methodsFor: 'protocol' stamp: ''!
entryCompletion: anEntryCompletion

	entryCompletion contents: anEntryCompletion.! !

!TextInputFieldModel methodsFor: 'protocol' stamp: ''!
ghostText

	^ ghostText contents! !

!TextInputFieldModel methodsFor: 'protocol' stamp: ''!
ghostText: aText

	ghostText contents: aText translated.! !

!TextInputFieldModel methodsFor: 'initialization' stamp: 'StephaneDucasse 5/17/2012 17:56'!
initialize
	"Initialization code for TextComposableModel"

	super initialize.
	
	ghostText := '' asValueHolder.
	acceptOnCR := true asValueHolder.
	autoAccept := false asValueHolder.
	entryCompletion := self defaultEntryCompletion asValueHolder.
	
	ghostText  whenChangedDo: [:txt | widget ifNotNil: [ widget ghostText: txt ]].
	acceptOnCR whenChangedDo: [:bool | widget ifNotNil: [ widget acceptOnCR: bool ]].
	autoAccept whenChangedDo: [:bool | widget ifNotNil: [ widget autoAccept: bool ]].! !

!TextInputFieldModel methodsFor: 'protocol' stamp: ''!
removeEntryCompletion

	self entryCompletion: nil! !

!TextInputFieldModel methodsFor: 'accessing' stamp: ''!
textHolder

	^ textHolder! !

!TextModel class methodsFor: 'example' stamp: ''!
example
	| instance |
	
	instance := self new.
	instance openWithSpec.
	instance text: 'This is a test'.! !

!TextModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 4/24/2012 03:31'!
internSpec
	<spec>
	^ {#TextSpec.
	    		#on:text:accept:readSelection:menu:. 	#model. #getText. #accept:notifying:. #readSelection. #codePaneMenu:shifted:.
			#classOrMetaClass:.				{ 	#model. 	#behavior}.
			#enabled:.							{	#model.	#enabled }.
			#hResizing:. 						#spaceFill.
			#vResizing:. 						#spaceFill.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!TextModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Text'! !

!TextModel methodsFor: 'protocol-shout' stamp: ''!
aboutToStyle: aBoolean	

	aboutToStyleHolder contents: [ aBoolean ]! !

!TextModel methodsFor: 'protocol-shout' stamp: 'StephaneDucasse 5/17/2012 17:43'!
aboutToStyleBlock

	^ aboutToStyleHolder contents! !

!TextModel methodsFor: 'protocol-shout' stamp: ''!
aboutToStyleBlock: aBlockReturningABoolean	

	aboutToStyleHolder contents: aBlockReturningABoolean! !

!TextModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 17:50'!
accept

	widget ifNotNil: [ widget accept ]! !

!TextModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 17:45'!
accept: aText notifying: aNotifyier

	textHolder contents: aText.
	actionToPerformHolder contents cull: aText cull: aNotifyier.
	self changed: #clearUserEdits.! !

!TextModel methodsFor: 'protocol' stamp: ''!
action: aBlockWithOneArgument

	actionToPerformHolder contents: aBlockWithOneArgument! !

!TextModel methodsFor: 'protocol-shout' stamp: 'StephaneDucasse 5/17/2012 17:40'!
behavior
	"Return the class corresponding to the method class of the source code you are editing"
	^ behaviorHolder contents! !

!TextModel methodsFor: 'protocol-shout' stamp: ''!
behavior: aText

	behaviorHolder contents: aText! !

!TextModel methodsFor: 'morphic' stamp: ''!
clearUserEditFlag
	
	self text: self getText.
	self changed: #clearUserEdits! !

!TextModel methodsFor: 'morphic' stamp: ''!
codePaneMenu: aMenu shifted: shifted 
	"Note that unless we override perform:orSendTo:, 
	PluggableTextController will respond to all menu items in a 
	text pane"
	
	^ shifted
			ifTrue: [SmalltalkEditor shiftedYellowButtonMenu]
			ifFalse: [SmalltalkEditor yellowButtonMenu].! !

!TextModel methodsFor: 'morphic' stamp: ''!
doItContext

	^ nil! !

!TextModel methodsFor: 'morphic' stamp: ''!
doItReceiver

	^ self selectedBehavior
		ifNil: [ nil ]
		ifNotNil: [:class | class theNonMetaClass ]! !

!TextModel methodsFor: 'protocol' stamp: ''!
enabled

	^ enabledHolder contents! !

!TextModel methodsFor: 'protocol' stamp: ''!
enabled: aBoolean

	enabledHolder contents: aBoolean! !

!TextModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 17:47'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<Ctrl-Cmd-right>'! !

!TextModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 17:47'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"

	^ '<Ctrl-Cmd-left>'! !

!TextModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 17:46'!
getText
	
	^ textHolder contents! !

!TextModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 17:50'!
hasEditingConflicts

	^ widget 
		ifNil: [ false ]
		ifNotNil: [ widget hasEditingConflicts ]! !

!TextModel methodsFor: 'initialization' stamp: 'StephaneDucasse 5/17/2012 17:50'!
initialize
	"Initialization code for TextComposableModel"

	super initialize.
	
	textHolder := '' asValueHolder.
	behaviorHolder := nil asValueHolder.
	aboutToStyleHolder := [ false ] asValueHolder.
	actionToPerformHolder := [ :text |  ] asValueHolder.
	readSelectionHolder := [ 1 to: 0 ] asValueHolder.
	enabledHolder := true asValueHolder.
	
	self registerEvents.
	self registerEventsForShout! !

!TextModel methodsFor: 'protocol-shout' stamp: 'StephaneDucasse 5/17/2012 17:43'!
isAboutToStyle

	^ self aboutToStyleBlock value! !

!TextModel methodsFor: 'private-shout' stamp: ''!
okToStyle

	^ true! !

!TextModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 17:57'!
pendingText

	self widget
		ifNil: [ ^ self getText ]
		ifNotNil: [ ^ widget text ]! !

!TextModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 17:57'!
pendingText: aText

	self widget
		ifNil: [ self text: aText ]
		ifNotNil: [ 
			widget setText: aText.
			widget hasUnacceptedEdits: true ]! !

!TextModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 17:50'!
readSelection

	^ readSelectionHolder contents cull: (widget ifNil: [ nil ] ifNotNil: [ widget text ]) cull: self! !

!TextModel methodsFor: 'protocol' stamp: ''!
readSelectionBlock

	^ readSelectionHolder contents! !

!TextModel methodsFor: 'protocol' stamp: ''!
readSelectionBlock: aBlock

	^ readSelectionHolder contents: aBlock! !

!TextModel methodsFor: 'initialization' stamp: 'StephaneDucasse 5/17/2012 17:50'!
registerEvents
	textHolder whenChangedDo: [ self changed: #getText ].
	readSelectionHolder whenChangedDo: [ self changed: #readSelection ].
	enabledHolder whenChangedDo: [ :bool | widget ifNotNil: [ widget enabled: bool ] ]! !

!TextModel methodsFor: 'initialization' stamp: 'StephaneDucasse 5/17/2012 17:50'!
registerEventsForShout
	aboutToStyleHolder whenChangedDo: [ self changed: #getText ].
	behaviorHolder
		whenChangedDo: [ :class | 
			widget ifNotNil: [ :mrph | mrph classOrMetaClass: class ].
			self changed: #getText ]! !

!TextModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 17:45'!
selectedBehavior

	^ behaviorHolder contents! !

!TextModel methodsFor: 'protocol' stamp: 'StephaneDucasse 5/17/2012 17:57'!
setSelection: anInterval

	self widget setSelection: anInterval! !

!TextModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 17:44'!
shoutAboutToStyle: aMorph
	
	^ self isAboutToStyle! !

!TextModel methodsFor: 'protocol' stamp: ''!
text: aText

	textHolder contents: aText! !

!TextModel methodsFor: 'protocol-events' stamp: ''!
whenTextChanges: aBlock

	textHolder whenChangedDo: aBlock! !

ComposableModel subclass: #TreeModel
	instanceVariableNames: 'childrenBlockHolder rootsHolder selectionHolder menuBlockHolder wrapBlockHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!TreeModel commentStamp: '<historical>' prior: 0!
A TreeModel is a spec model of a Tree ;)!

!TreeModel class methodsFor: 'example' stamp: 'BenjaminVanRyseghem 4/13/2012 14:39'!
example
	| m |

	m := TreeModel new.
	m openWithSpec.
	m roots: #(a b c d).
	m childrenBlock: [:it | it = #c 
							ifTrue: [ {1. 2. 3} ]
							ifFalse: [ {} ]].
	m wrappingBlock: [:item | item isSymbol
								ifTrue: [ item, ' symbol' ]
								ifFalse: [ item printString ]].! !

!TreeModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 4/24/2012 03:17'!
internSpec
	<spec>
	
	^ {#TreeSpec.
	    		#model:.								#model.
			#getRootsSelector:.				#getRoots.
			#getChildrenSelector:.				#getChildrenFor:.
			#setSelectedSelector:.				#setSelectedItem:.
			#getMenuSelector:.				#getMenu:.
			#getLabelSelector:.				#wrapItem:.
			#hResizing:. 						#spaceFill.
			#vResizing:. 						#spaceFill.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!TreeModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 4/13/2012 14:38'!
title

	^ 'Tree'! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/13/2012 14:31'!
childrenBlock: aBlock

	childrenBlockHolder contents: aBlock! !

!TreeModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<right>'! !

!TreeModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!TreeModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 4/13/2012 14:24'!
getChildrenFor: anItem

		^ childrenBlockHolder contents cull: anItem cull: self! !

!TreeModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 4/13/2012 14:29'!
getMenu: aMenu

	menuBlockHolder contents cull: aMenu cull: self.
	^ aMenu! !

!TreeModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 4/13/2012 14:22'!
getRoots
							
		^ rootsHolder contents! !

!TreeModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 4/13/2012 14:36'!
initialize
	"Initialization code for TreeModel"

	super initialize.

	childrenBlockHolder := [ {} ] asValueHolder.
	rootsHolder := {} asValueHolder.
	selectionHolder := nil asValueHolder.
	menuBlockHolder := [] asValueHolder.
	wrapBlockHolder := [:item | item printString ] asValueHolder.

	self registerEvents! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/13/2012 14:40'!
menu: aBlock

	menuBlockHolder contents: aBlock! !

!TreeModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 4/13/2012 14:40'!
registerEvents

	rootsHolder whenChangedDo: [ self changed: #getRoots ].
	childrenBlockHolder whenChangedDo: [ self changed: #getRoots ].
	wrapBlockHolder whenChangedDo: [ self changed: #getRoots ].! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/13/2012 14:28'!
roots: aCollection

	rootsHolder contents: aCollection! !

!TreeModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 4/13/2012 14:30'!
setSelectedItem: anItem

	selectionHolder contents: anItem.! !

!TreeModel methodsFor: 'protocol events' stamp: 'StephaneDucasse 5/17/2012 19:32'!
whenSelectedItemChanged: aBlock

	selectionHolder whenChangedDo: aBlock! !

!TreeModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 4/13/2012 14:35'!
wrapItem: anItem

	^ wrapBlockHolder contents cull: anItem cull: self! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/13/2012 14:39'!
wrappingBlock: aBlock

	wrapBlockHolder contents: aBlock! !
